/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.common.command.grpc;

import java.time.ZonedDateTime;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.NullValueCheckStrategy;

import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.OzgCloudCommandId;
import de.ozgcloud.apilib.common.command.OzgCloudCommandStatus;
import de.ozgcloud.apilib.common.command.OzgCloudCreateSubCommandsRequest;
import de.ozgcloud.apilib.common.datatypes.GenericId;
import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.common.datatype.StringBasedValue;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.grpc.command.GrpcAddSubCommandsRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcCreateCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcCreateCommandRequest;

@Mapper(uses = { GrpcObjectMapper.class }, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface CommandMapper {

	@Mapping(target = "redirectRequestOrBuilder", ignore = true)
	@Mapping(target = "orderStringBytes", ignore = true)
	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "callContextOrBuilder", ignore = true)
	@Mapping(target = "bodyObjOrBuilder", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeBodyObj", ignore = true)
	@Mapping(target = "mergeCallContext", ignore = true)
	@Mapping(target = "mergeRedirectRequest", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "removeBody", ignore = true)
	@Mapping(target = "callContext", ignore = true)
	@Mapping(target = "redirectRequest", ignore = true)
	@Mapping(target = "relationIdBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "vorgangIdBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "bodyBuilderList", ignore = true)
	@Mapping(target = "bodyList", ignore = true)
	@Mapping(target = "bodyOrBuilderList", ignore = true)

	@Mapping(target = "bodyObj", source = "bodyObject")
	@Mapping(target = "orderString", source = "order")
	GrpcCreateCommandRequest buildCreateCommandRequest(OzgCloudCommand commandToCreate);

	@Mapping(target = "bodyObject", source = "bodyObj")
	@Mapping(target = "order", source = "orderString")
	@Mapping(target = "relationVersion", ignore = true)
	@Mapping(target = "relationId", qualifiedByName = "mapRelationId")
	@Mapping(target = "body", ignore = true)
	@Mapping(target = "createdAt", expression = "java(mapToZonedDateTime(command.getCreatedAt()))")
	@Mapping(target = "finishedAt", expression = "java(mapToZonedDateTime(command.getFinishedAt()))")
	OzgCloudCommand fromGrpc(GrpcCommand command);

	default ZonedDateTime mapToZonedDateTime(String value) {
		return StringUtils.isEmpty(value) ? null : ZonedDateTime.parse(value);
	}

	@Named("mapRelationId")
	default StringBasedValue mapRelationId(String relationId) {
		return GenericId.from(relationId);
	}

	default OzgCloudVorgangId toOzgCloudVorgangId(String userId) {
		return OzgCloudVorgangId.from(userId);
	}

	default OzgCloudUserId toOzgCloudUserId(String userId) {
		return OzgCloudUserId.from(userId);
	}

	default OzgCloudCommandStatus toOzgCloudCommandStatus(String status) {
		return OzgCloudCommandStatus.from(status);
	}

	default OzgCloudCommandId toOzgCloudCommandId(String id) {
		return OzgCloudCommandId.from(id);
	}

	default String valueToString(StringBasedValue value) {
		return value.toString();
	}

	default GrpcAddSubCommandsRequest toAddSubCommandsRequest(OzgCloudCreateSubCommandsRequest createSubCommandsRequest) {
		return toAddSubCommandsRequestWithoutSubCommands(createSubCommandsRequest).toBuilder()
			.addAllSubCommands(toCreateCommands(createSubCommandsRequest.getSubCommands())).build();
	}

	@Mapping(target = "subCommandsList", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "subCommandsOrBuilderList", ignore = true)
	@Mapping(target = "subCommandsBuilderList", ignore = true)
	@Mapping(target = "removeSubCommands", ignore = true)
	@Mapping(target = "parentIdBytes", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "executionModeBytes", ignore = true)
	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	GrpcAddSubCommandsRequest toAddSubCommandsRequestWithoutSubCommands(OzgCloudCreateSubCommandsRequest command);

	default List<GrpcCreateCommand> toCreateCommands(List<OzgCloudCommand> subCommands) {
		return subCommands.stream().map(this::toCreateCommand).toList();
	}

	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "relationIdBytes", ignore = true)
	@Mapping(target = "orderBytes", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "mergeBodyObj", ignore = true)
	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "bodyObjOrBuilder", ignore = true)
	@Mapping(target = "allFields", ignore = true)

	@Mapping(target = "bodyObj", source = "bodyObject")
	GrpcCreateCommand toCreateCommand(OzgCloudCommand commands);

}
