/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.apilib.vorgang.grpc;

import java.util.List;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;

import de.ozgcloud.apilib.vorgang.OzgCloudPostfachAddress;
import de.ozgcloud.apilib.vorgang.OzgCloudPostfachAddressIdentifier;
import de.ozgcloud.apilib.vorgang.OzgCloudServiceKonto;
import de.ozgcloud.apilib.vorgang.OzgCloudServiceKontoType;
import de.ozgcloud.apilib.vorgang.OzgCloudTrustLevel;
import de.ozgcloud.vorgang.common.GrpcObject;
import de.ozgcloud.vorgang.common.GrpcProperty;
import de.ozgcloud.vorgang.vorgang.GrpcPostfachAddress;
import de.ozgcloud.vorgang.vorgang.GrpcServiceKonto;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangHead;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Mapper(nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public abstract class OzgCloudServiceKontoMapper {

	OzgCloudServiceKonto getServiceKonto(GrpcVorgangHead header) {
		return header.hasServiceKonto() ? fromServiceKonto(header.getServiceKonto()) : null;
	}

	@Mapping(target = "address", source = "postfachAddressesList")
	abstract OzgCloudServiceKonto fromServiceKonto(GrpcServiceKonto serviceKonto);

	OzgCloudPostfachAddress fromPostfachAddresses(List<GrpcPostfachAddress> postfachAddresses) {
		if (postfachAddresses.isEmpty()) {
			return null;
		}
		if (postfachAddresses.size() > 1) {
			LOG.error("More than one postfach address found. Using the first one.");
		}
		return fromPostfachAddress(postfachAddresses.getFirst());
	}

	@Mapping(target = "identifier", ignore = true)
	@Mapping(target = "identifiers", source = "identifier")
	abstract OzgCloudPostfachAddress fromPostfachAddress(GrpcPostfachAddress postfachAddress);

	List<OzgCloudPostfachAddressIdentifier> fromPostfachAddressIdentifier(GrpcObject identifier) {
		if (!identifier.getSubObjectList().isEmpty()) {
			LOG.error("Unexpected subObjects found in identifier. Ignoring them.");
		}
		return identifier.getPropertyList().stream().map(this::fromProperty).toList();
	}

	OzgCloudPostfachAddressIdentifier fromProperty(GrpcProperty grpcProperty) {
		return OzgCloudPostfachAddressIdentifier.builder()
			.name(grpcProperty.getName())
			.value(grpcProperty.getValueList().getFirst())
			.build();
	}

	OzgCloudServiceKontoType toOzgCloudServiceKontoType(String type) {
		return Optional.ofNullable(type).map(StringUtils::trimToNull).map(OzgCloudServiceKontoType::from).orElse(null);
	}

	OzgCloudTrustLevel toOzgCloudTrustLevel(String trustLevel) {
		return Optional.ofNullable(trustLevel).map(StringUtils::trimToNull).map(OzgCloudTrustLevel::from).orElse(null);
	}

}
