/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.vorgang.grpc;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Optional;
import java.util.Set;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.apilib.vorgang.FormIdentificationTestFactory;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQuery;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQuery.FormIdentification;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQuery.Query;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangQueryTestFactory;
import de.ozgcloud.apilib.vorgang.QueryTestFactory;
import de.ozgcloud.vorgang.common.GrpcQueryOperator;
import de.ozgcloud.vorgang.vorgang.GrpcQuery;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangQueryExpression;

class OzgCloudVorgangQueryMapperTest {

	@Spy
	private final OzgCloudVorgangQueryMapper mapper = Mappers.getMapper(OzgCloudVorgangQueryMapper.class);

	@Nested
	class TestToGrpc {

		@Mock
		private GrpcQuery grpcQuery;

		@Test
		void shouldCallHasPropertyExpressions() {
			var ozgCloudVorgangQuery = OzgCloudVorgangQueryTestFactory.create();

			mapper.toGrpc(ozgCloudVorgangQuery);

			verify(mapper).hasPropertyExpressions(ozgCloudVorgangQuery);
		}

		@Nested
		class TestHasPropertyExpressions {

			@BeforeEach
			void init() {
				doReturn(true).when(mapper).hasPropertyExpressions(any());
			}

			@Test
			void shouldCallMapToQuery() {
				var ozgCloudVorgangQuery = OzgCloudVorgangQueryTestFactory.create();

				mapper.toGrpc(ozgCloudVorgangQuery);

				verify(mapper).mapAllToQuery(ozgCloudVorgangQuery);
			}

			@Test
			void shouldReturnResult() {
				doReturn(grpcQuery).when(mapper).mapAllToQuery(any());

				var result = mapper.toGrpc(OzgCloudVorgangQueryTestFactory.create());

				assertThat(result).isSameAs(grpcQuery);
			}
		}

		@Nested
		class TestHasNoPropertyExpressions {

			@BeforeEach
			void init() {
				doReturn(false).when(mapper).hasPropertyExpressions(any());
			}

			@Test
			void shouldCallToQuery() {
				var ozgCloudVorgangQuery = OzgCloudVorgangQueryTestFactory.create();

				mapper.toGrpc(ozgCloudVorgangQuery);

				verify(mapper).toQuery(OzgCloudVorgangQueryTestFactory.QUERY);
			}

			@Test
			void shouldReturnResult() {
				doReturn(grpcQuery).when(mapper).toQuery(any(Query.class));

				var result = mapper.toGrpc(OzgCloudVorgangQueryTestFactory.create());

				assertThat(result).isSameAs(grpcQuery);
			}

			@Test
			void shouldReturnDefaultInstance() {
				var ozgCloudQuery = OzgCloudVorgangQueryTestFactory.createBuilder().query(Optional.empty()).build();

				var result = mapper.toGrpc(ozgCloudQuery);

				assertThat(result).isEqualTo(GrpcQuery.getDefaultInstance());
			}
		}
	}

	@Nested
	class TestHasPropertyExpressions {

		@Test
		void shouldReturnFalse() {
			var query = OzgCloudVorgangQuery.builder().query(Optional.of(OzgCloudVorgangQueryTestFactory.QUERY)).build();

			var result = mapper.hasPropertyExpressions(query);

			assertThat(result).isFalse();
		}

		@Nested
		class TestOnPropertySet {

			@Test
			void shouldReturnTrueWithAktenzeichen() {
				var query = OzgCloudVorgangQuery.builder().aktenzeichen(Optional.of(OzgCloudVorgangQueryTestFactory.AKTENZEICHEN)).build();

				var result = mapper.hasPropertyExpressions(query);

				assertThat(result).isTrue();
			}

			@Test
			void shouldReturnTrueWithOrganisationsEinheitId() {
				var query = OzgCloudVorgangQuery.builder()
						.organisationsEinheitId(Optional.of(OzgCloudVorgangQueryTestFactory.ORGANISATIONS_EINHEIT_ID)).build();

				var result = mapper.hasPropertyExpressions(query);

				assertThat(result).isTrue();
			}

			@Test
			void shouldReturnTrueWithForm() {
				var query = OzgCloudVorgangQuery.builder().form(Optional.of(FormIdentificationTestFactory.create())).build();

				var result = mapper.hasPropertyExpressions(query);

				assertThat(result).isTrue();
			}

			@Test
			void shouldReturnTrueWithStatus() {
				var query = OzgCloudVorgangQuery.builder().status(Optional.of(OzgCloudVorgangQueryTestFactory.STATUS)).build();

				var result = mapper.hasPropertyExpressions(query);

				assertThat(result).isTrue();
			}
		}
	}

	@Nested
	class TestMapPropertiesToQuery {

		@Nested
		class TestForAktenzeichen {

			private final GrpcVorgangQueryExpression aktenzeichenExpression = GrpcVorgangQueryExpressionTestFactory.createBuilder()
					.setPath(OzgCloudVorgangQueryMapper.AKTENZEICHEN_PATH)
					.setOperator(GrpcQueryOperator.EQUAL)
					.setOperandStringValue(OzgCloudVorgangQueryTestFactory.AKTENZEICHEN)
					.build();

			@Test
			void shouldCallToAktenzeichenExpression() {
				mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.create());

				verify(mapper).toAktenzeichenExpression(OzgCloudVorgangQueryTestFactory.AKTENZEICHEN);
			}

			@Test
			void shouldSetAktenzeichenExpression() {
				doReturn(aktenzeichenExpression).when(mapper).toAktenzeichenExpression(OzgCloudVorgangQueryTestFactory.AKTENZEICHEN);

				var queryBuidler = mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.create());

				assertThat(queryBuidler.getExpressionsList()).contains(aktenzeichenExpression);
			}

			@Test
			void shouldNotCallToAktenzeichenExpression() {
				mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.createBuilder().aktenzeichen(Optional.empty()).build());

				verify(mapper, never()).toAktenzeichenExpression(any());
			}
		}

		@Nested
		class TestForOrganisationsEinheitId {

			private final GrpcVorgangQueryExpression organisationsEinheitIdExpression = GrpcVorgangQueryExpressionTestFactory.createBuilder()
					.setPath(OzgCloudVorgangQueryMapper.ORGANISATIONS_EINHEIT_ID_PATH)
					.setOperator(GrpcQueryOperator.EQUAL)
					.setOperandStringValue(OzgCloudVorgangQueryTestFactory.ORGANISATIONS_EINHEIT_ID)
					.build();

			@Test
			void shouldCallToOrganisationsEinheitIdExpression() {
				mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.create());

				verify(mapper).toOrganisationsEinheitIdExpression(OzgCloudVorgangQueryTestFactory.ORGANISATIONS_EINHEIT_ID);
			}

			@Test
			void shouldSetOrganisationsEinheitIdExpression() {
				doReturn(organisationsEinheitIdExpression).when(mapper)
						.toOrganisationsEinheitIdExpression(OzgCloudVorgangQueryTestFactory.ORGANISATIONS_EINHEIT_ID);

				var queryBuidler = mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.create());

				assertThat(queryBuidler.getExpressionsList()).contains(organisationsEinheitIdExpression);
			}

			@Test
			void shouldNotCallToOrganisationsEinheitIdExpression() {
				mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.createBuilder().organisationsEinheitId(Optional.empty()).build());

				verify(mapper, never()).toOrganisationsEinheitIdExpression(any());
			}
		}

		@Nested
		class TestForForm {

			private final GrpcVorgangQueryExpression formEngineNameExpression = GrpcVorgangQueryExpressionTestFactory.createBuilder()
					.setPath(OzgCloudVorgangQueryMapper.FORM_ENGINE_NAME_PATH)
					.setOperator(GrpcQueryOperator.EQUAL)
					.setOperandStringValue(FormIdentificationTestFactory.FORM_ENGINE_NAME)
					.build();

			private final Set<GrpcVorgangQueryExpression> formExpressions = Set.of(formEngineNameExpression);

			@Test
			void shouldCallToFormExpression() {
				mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.create());

				verify(mapper).toFormExpressions(OzgCloudVorgangQueryTestFactory.FORM);
			}

			@Test
			void shouldSetFormExpression() {
				doReturn(formExpressions).when(mapper).toFormExpressions(OzgCloudVorgangQueryTestFactory.FORM);

				var queryBuidler = mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.create());

				assertThat(queryBuidler.getExpressionsList()).containsAll(formExpressions);
			}

			@Test
			void shouldNotCallToFormExpression() {
				mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.createBuilder().form(Optional.empty()).build());

				verify(mapper, never()).toFormExpressions(any());
			}

			@Test
			void shouldNotSetEmptyFormExpressions() {
				doReturn(Collections.emptySet()).when(mapper).toFormExpressions(any());
				var ozgCloudVorgangQuery = OzgCloudVorgangQuery.builder().form(Optional.of(FormIdentification.builder().build())).build();

				var result = mapper.mapAllToQuery(ozgCloudVorgangQuery);

				assertThat(result.getExpressionsList()).isEmpty();
			}
		}

		@Nested
		class TestForStatus {

			private final GrpcVorgangQueryExpression statusExpression = GrpcVorgangQueryExpressionTestFactory.createBuilder()
					.setPath(OzgCloudVorgangQueryMapper.STATUS_PATH)
					.setOperator(GrpcQueryOperator.EQUAL)
					.setOperandStringValue(OzgCloudVorgangQueryTestFactory.STATUS)
					.build();

			@Test
			void shouldCallToStatusExpression() {
				mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.create());

				verify(mapper).toStatusExpression(OzgCloudVorgangQueryTestFactory.STATUS);
			}

			@Test
			void shouldSetStatusExpression() {
				doReturn(statusExpression).when(mapper)
						.toStatusExpression(OzgCloudVorgangQueryTestFactory.STATUS);

				var queryBuidler = mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.create());

				assertThat(queryBuidler.getExpressionsList()).contains(statusExpression);
			}

			@Test
			void shouldNotCallToStatusExpression() {
				mapper.mapAllToQuery(OzgCloudVorgangQueryTestFactory.createBuilder().status(Optional.empty()).build());

				verify(mapper, never()).toStatusExpression(any());
			}
		}

	}

	@Nested
	class TestToAktenzeichenExpression {

		private final GrpcVorgangQueryExpression aktenzeichenExpression = GrpcVorgangQueryExpressionTestFactory.createBuilder()
				.setPath(OzgCloudVorgangQueryMapper.AKTENZEICHEN_PATH)
				.setOperator(GrpcQueryOperator.EQUAL)
				.setOperandStringValue(OzgCloudVorgangQueryTestFactory.AKTENZEICHEN)
				.build();

		@Test
		void shouldReturnAktenzeichenExpression() {
			var expression = mapper.toAktenzeichenExpression(OzgCloudVorgangQueryTestFactory.AKTENZEICHEN);

			assertThat(expression).usingRecursiveComparison().isEqualTo(aktenzeichenExpression);
		}
	}

	@Nested
	class TestToOrganisationsEinheitIdExpression {

		private final GrpcVorgangQueryExpression organisationsEinheitIdExpression = GrpcVorgangQueryExpressionTestFactory.createBuilder()
				.setPath(OzgCloudVorgangQueryMapper.ORGANISATIONS_EINHEIT_ID_PATH)
				.setOperator(GrpcQueryOperator.EQUAL)
				.setOperandStringValue(OzgCloudVorgangQueryTestFactory.ORGANISATIONS_EINHEIT_ID)
				.build();

		@Test
		void shouldReturnOrganisationsEinheitIdExpression() {
			var expression = mapper.toOrganisationsEinheitIdExpression(OzgCloudVorgangQueryTestFactory.ORGANISATIONS_EINHEIT_ID);

			assertThat(expression).usingRecursiveComparison().isEqualTo(organisationsEinheitIdExpression);
		}
	}

	@Nested
	class TestToStatusExpression {

		private final GrpcVorgangQueryExpression statusExpression = GrpcVorgangQueryExpressionTestFactory.createBuilder()
				.setPath(OzgCloudVorgangQueryMapper.STATUS_PATH)
				.setOperator(GrpcQueryOperator.EQUAL)
				.setOperandStringValue(OzgCloudVorgangQueryTestFactory.STATUS)
				.build();

		@Test
		void shouldReturnStatusExpression() {
			var expression = mapper.toStatusExpression(OzgCloudVorgangQueryTestFactory.STATUS);

			assertThat(expression).usingRecursiveComparison().isEqualTo(statusExpression);
		}
	}

	@Nested
	class TestToFormExpressions {

		@Mock
		private GrpcVorgangQueryExpression formEngineNameExpression;

		@Mock
		private GrpcVorgangQueryExpression formIdExpression;

		@Test
		void shouldCallToFormEngineNameExpression() {
			toFormExpressions();

			verify(mapper).toFormEngineNameExpression(FormIdentificationTestFactory.FORM_ENGINE_NAME);
		}

		@Test
		void shouldSetFormEngineNameExpression() {
			doReturn(formEngineNameExpression).when(mapper).toFormEngineNameExpression(any());

			var expressions = toFormExpressions();

			assertThat(expressions).contains(formEngineNameExpression);
		}

		@Test
		void shouldCallToFormIdExpression() {
			toFormExpressions();

			verify(mapper).toFormIdExpression(FormIdentificationTestFactory.FORM_ID);
		}

		@Test
		void shouldReturnFormIdExpression() {
			doReturn(formIdExpression).when(mapper).toFormIdExpression(any());

			var expressions = toFormExpressions();

			assertThat(expressions).contains(formIdExpression);
		}

		private Set<GrpcVorgangQueryExpression> toFormExpressions() {
			return mapper.toFormExpressions(FormIdentificationTestFactory.create());
		}
	}

	@Nested
	class TestToFormEngineNameExpression {

		@Test
		void shouldSetPath() {
			var expression = mapper.toFormEngineNameExpression(FormIdentificationTestFactory.FORM_ENGINE_NAME);

			assertThat(expression.getPath()).isEqualTo(OzgCloudVorgangQueryMapper.FORM_ENGINE_NAME_PATH);
		}

		@Test
		void shouldSetOperandStringValue() {
			var expression = mapper.toFormEngineNameExpression(FormIdentificationTestFactory.FORM_ENGINE_NAME);

			assertThat(expression.getOperandStringValue()).isEqualTo(FormIdentificationTestFactory.FORM_ENGINE_NAME);
		}

		@Test
		void shouldSetOperator() {
			var expression = mapper.toFormEngineNameExpression(FormIdentificationTestFactory.FORM_ENGINE_NAME);

			assertThat(expression.getOperator()).isEqualTo(GrpcQueryOperator.EQUAL);
		}
	}

	@Nested
	class TestToFormIdExpression {

		@Test
		void shouldSetPath() {
			var expression = mapper.toFormIdExpression(FormIdentificationTestFactory.FORM_ID);

			assertThat(expression.getPath()).isEqualTo(OzgCloudVorgangQueryMapper.FORM_ID_PATH);
		}

		@Test
		void shouldSetOperandStringValue() {
			var expression = mapper.toFormIdExpression(FormIdentificationTestFactory.FORM_ID);

			assertThat(expression.getOperandStringValue()).isEqualTo(FormIdentificationTestFactory.FORM_ID);
		}

		@Test
		void shouldSetOperator() {
			var expression = mapper.toFormIdExpression(FormIdentificationTestFactory.FORM_ID);

			assertThat(expression.getOperator()).isEqualTo(GrpcQueryOperator.EQUAL);
		}
	}

	@Nested
	class TestToQuery {

		private final Query nestedQuery = QueryTestFactory.create();
		private final Query query = QueryTestFactory.createBuilder().nestedQuery(nestedQuery).build();

		private final GrpcQuery nestedGrpcQuery = GrpcQueryTestFactory.create();
		private final GrpcQuery expectedGrpcQuery = GrpcQueryTestFactory.createBuilder().setNestedQuery(nestedGrpcQuery).build();

		@Test
		void shouldMapQuery() {
			var grpcQuery = mapper.toQuery(query);

			assertThat(grpcQuery).usingRecursiveComparison().isEqualTo(expectedGrpcQuery);
		}
	}
}
